/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;                                                                     
;                     Software License Agreement                      
;                                                                     
; The software supplied herewith by Microchip Technology Incorporated 
; (the "Company") for its PICmicro Microcontroller is intended and   
; supplied to you, the Companys customer, for use solely and         
; exclusively on Microchip PICmicro Microcontroller products.         
;                                                                     
; The software is owned by the Company and/or its supplier, and is     
; protected under applicable copyright laws. All rights are reserved.  
; Any use in violation of the foregoing restrictions may subject the  
; user to criminal sanctions under applicable laws, as well as to     
; civil liability for the breach of the terms and conditions of this  
; license.                                                             
;                                                                      
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR          
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                    
;                                                                     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	Filename:			    AN999.c            
;	Date:				      June 14, 2005          
;	File Version:		  1.0                               
;	Assembled using:	MPLAB IDE 7.00.00.0               
; Author:		  	    Martin Kvasnicka              
;	Company:			    Microchip Technology, Inc.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#include <p18f452.h>
#include "AN995.h"
#include <spi.h>

//config words
#pragma romdata CONFIG
rom const unsigned char CONFIG1L = 0xFF;	//Dummy byte
rom const unsigned char CONFIG1H = 0xFA;	//HS + 4x PLL mode off, OSC switch enabled
rom const unsigned char CONFIG2L = 0xFC;	//BOR disabled, PWRTE enabled
rom const unsigned char CONFIG2H = 0x00;	//WDT disabled
rom const unsigned char CONFIG3L = 0xFF;	//Dummy byte
rom const unsigned char CONFIG3H = 0xFF;	//CCP is on RC1
rom const unsigned char CONFIG4L = 0xFB;	//Debug off, LVP off, stack reset
rom const unsigned char CONFIG4H = 0xFF;	//Dummy byte
rom const unsigned char CONFIG5L = 0xFF;	//Code protection off
rom const unsigned char CONFIG5H = 0xFF;	//EE unprotect, Boot unprotect
rom const unsigned char CONFIG6L = 0xFF;	//Write protection off
rom const unsigned char CONFIG6H = 0xFF;	//EE unprotect, Boot unprotect, Config unprotect
rom const unsigned char CONFIG7L = 0xFF;	//Reads unprotected
rom const unsigned char CONFIG7H = 0xFF;	//Boot read unprotected
#pragma romdata

//********************************************************************
// Global Variable declarations
//********************************************************************

unsigned char HighAdd;                    //High Order Address Byte
unsigned char LowAdd;                     //Low Order Address Byte
unsigned char Data;                       //Data Byte
unsigned char Length;                     //Length of bytes to read
unsigned int  PageSize;                   //Page size in bytes
unsigned char PageString[64];             // Holds the device page data to/from EEPROM

void main(void)
{
  unsigned int n;
  HighAdd = 0x00;                         //HighAdd is first byte transmitted for High Density devices
  LowAdd = 0x10;                          //LowAdd is only Address byte for Low Density, second byte to HD
  Data = 0xA5;                            //Data can be whatever user would like
  Length = 0x01;                          //Length param set to 1 for single byte read
  PageSize = 0x40;                        //PageSize is Physical page boundry from Datasheet. (0x10 = 16, 0x20 = 32)
  for(n = 0; n < PageSize; n++)
  {
    PageString[n] = Data;                 //Fill PageString with data to be written
  }

  InitPIC();
   
//  LDByteWriteSPI( LowAdd, Data );                                 //Low Density Byte Write
//  LDByteReadSPI(LowAdd, &Data, Length );                          //Low Density Byte Read
//  LDPageWriteSPI(LowAdd, PageString );                            //Low Density Page Write
//  LDSequentialReadSPI(LowAdd, PageString, PageSize );             //Low Density Page Read

//  HDByteWriteSPI(HighAdd, LowAdd, Data );                         //High Density Byte Write
//  HDByteReadSPI(HighAdd, LowAdd , &Data, Length) ;                //High Density Byte Read
//  HDPageWriteSPI(HighAdd, LowAdd, PageString );                   //High Density Page Write
//  HDSequentialReadSPI(HighAdd, LowAdd, PageString, PageSize );    //High Density Page Read  
}

void InitPIC(void)
//here is the SPI setup
{
  DDRCbits.RC7 = 0;       //Define CS as Output
	DDRCbits.RC3 = 0;       //Define SCK as Output
  DDRCbits.RC4 = 1;       //Define SDI as Input
  DDRCbits.RC5 = 0;       //Define SDO as Output
	SSPSTAT = 0xC0;         //SPI Bus mode 0,0
	SSPCON1 = 0x21;         //Enable SSP, Fosc/16
}
