#include <p18f4550.h>
#include <spi.h>
#include "AN995.h"

#define PageSize 8

/********************************************************************
*     Function Name:    LDByteWriteSPI                              *
*     Parameters:       EE memory control, address and pointer 1    *
*     Description:      Writes Data Byte to SPI EE memory device    *
*                       This routine can be used for any SPI        *   
*                       EE memory device with 1 byte of address.    *
*                                                                   *  
********************************************************************/
unsigned char LDByteWriteSPI(unsigned char LowAdd, unsigned char Data )
{
  WriteEnable();                      // Write Enable prior to Write
  ReadStatus();                       // Check for WEL bit set
  CS = 0;                             // Select Device
  WriteSPI ( 0x02 );                  // Send Write OpCode
  WriteSPI ( LowAdd );                // write address byte to EEPROM
  WriteSPI ( Data );                  // Write Byte to device
  CS = 1;                             // Deselect device and initiate Write
  SPIWIPPolling();                    // Wait for Write to complete
  return ( 0 );
}

/********************************************************************
*     Function Name:    LDByteReadSPI                               *
*     Parameters:       EE memory control, address, pointer and     *
*                       length bytes.                               *
*     Description:      Reads data Byte from SPI EE memory device.  *
*                       This routine can be used for any SPI        *
*                       EE memory device with 1 byte of address     *
*                                                                   *
********************************************************************/  
unsigned char LDByteReadSPI(unsigned char LowAdd, unsigned char *rdptr, unsigned char length )
{
  CS = 0;                             // Select Device
  WriteSPI( 0x03 );                   // Send Read OpCode 
  WriteSPI( LowAdd );                 // WRITE word address to EEPROM
  getsSPI( rdptr, length );           // read in multiple bytes
  CS = 1;                             // Deselect Device
  return ( 0 );                                 
}

/********************************************************************
*     Function Name:    LDPageWriteSPI                              *
*     Parameters:       EE memory control, address and pointer 1    *
*     Description:      Writes data string to SPI EE memory         *
*                       device. This routine can be used for any SPI*
*                       EE memory device with 1 byte of address.    *
*                                                                   *  
********************************************************************/
unsigned char LDPageWriteSPI( unsigned char LowAdd, unsigned char *wrptr )
{
  WriteEnable();                      // Write Enable prior to Write
  ReadStatus();                       // Check for WEL bit set
  CS = 0;                             // Select Device
  WriteSPI ( 0x02 );                  // send Write OpCode
  WriteSPI ( LowAdd );                // write address byte to EEPROM
  PutStringSPI ( wrptr );             // Write Page to device
  CS = 1;                             // Deselect Device
  SPIWIPPolling();                    // Wait for Write to Complete
  return ( 0 );
}

/********************************************************************
*     Function Name:    LDSequentialReadSPI                         *
*     Parameters:       EE memory control, address, pointer and     *
*                       length bytes.                               *
*     Description:      Reads data string from SPI EE memory        *
*                       device. This routine can be used for any SPI*
*                       EE memory device with 1 byte of address.    *
*                                                                   *
********************************************************************/  
unsigned char LDSequentialReadSPI( unsigned char LowAdd, unsigned char *rdptr, unsigned char length )
{
  CS = 0;                             // Select Device
  WriteSPI( 0x03 );                   // Send Read OpCode 
  WriteSPI( LowAdd );                 // WRITE word address to EEPROM
  getsSPI( rdptr, length );           // read in multiple bytes
  CS = 1;                             // Deselect Device    
  return ( 0 );                       
}

/********************************************************************
*     Function Name:    HDByteWriteSPI                              *
*     Return Value:     error condition status                      *
*     Parameters:       EE memory control, address and pointer 1    *
*     Description:      Writes data string to SPI EE memory         *
*                       device. This routine can be used for any SPI*
*                       EE memory device with two address bytes.    *
*                                                                   *  
********************************************************************/
unsigned char HDByteWriteSPI(unsigned char HighAdd, unsigned char LowAdd, unsigned char Data )
{
  WriteEnable();                      // Write Enable prior to Write
  ReadStatus();                       // Check for WEL bit set
  CS = 0;                             // Select Device
  WriteSPI ( 0x02 );                  // Send Write OpCode
  WriteSPI ( HighAdd);                // Send High Address Byte
  WriteSPI ( LowAdd );                // write address byte to EEPROM
  WriteSPI ( Data );                  // Write Byte to device
  CS = 1;                             // Deselect Device
  SPIWIPPolling();                    // Wait for Write to Complete
  return ( 0 );
}

/********************************************************************
*     Function Name:    HDByteReadSPI                               *
*     Parameters:       EE memory control, address, pointer and     *
*                       length bytes.                               *
*     Description:      Reads data string from SPI EE memory        *
*                       device. This routine can be used for any SPI*
*                       EE memory device with two address bytes.    *
*                                                                   *
********************************************************************/  
unsigned char HDByteReadSPI(unsigned char HighAdd, unsigned char LowAdd, unsigned char *rdptr, unsigned char length )
{
  CS = 0;                             // Select Device
  WriteSPI( 0x03 );                   // Send Read OpCode 
  WriteSPI( HighAdd);                 // Send High Address Byte
  WriteSPI( LowAdd );                 // WRITE word address to EEPROM
  getsSPI( rdptr, length );           // read in multiple bytes
  CS = 1;                             // Deselect Device
  return ( 0 );                       // return with no error
}

/********************************************************************
*     Function Name:    HDPageWriteSPI                                 *
*     Return Value:     error condition status                      *
*     Parameters:       EE memory control, address and pointer 1    *
*     Description:      Writes data string to SPI EE memory         *
*                       device. This routine can be used for any SPI*
*                       EE memory device with two address bytes.    *
*                                                                   *  
********************************************************************/
unsigned char HDPageWriteSPI( unsigned char HighAdd, unsigned char LowAdd, unsigned char *wrptr )
{
  WriteEnable();                      // Write Enable prior to Write
  ReadStatus();                       // Check for WEL bit set
  CS = 0;                             // Select Device
  WriteSPI ( 0x02 );                  // send Write OpCode
  WriteSPI ( HighAdd);                // Send High Address Byte
  WriteSPI ( LowAdd );                // write address byte to EEPROM
  PutStringSPI ( wrptr );             // Write Page to device
  CS = 1;                             // Deselect Device
  SPIWIPPolling();                    // Wait for Write to Complete
  return ( 0 );
}

/********************************************************************
*     Function Name:    HDSequentialReadSPI                         *
*     Parameters:       EE memory control, address, pointer and     *
*                       length bytes.                               *
*     Description:      Reads data string from SPI EE memory        *
*                       device. This routine can be used for any SPI*
*                       EE memory device with two address bytes.    *
*                                                                   *
********************************************************************/  
unsigned char HDSequentialReadSPI( unsigned char HighAdd, unsigned char LowAdd, unsigned char *rdptr, unsigned char length )
{
  CS = 0;                             // Select Device
  WriteSPI( 0x03 );                   // Send Read OpCode 
  WriteSPI (HighAdd);                 // Send High Address Byte
  WriteSPI( LowAdd );                 // WRITE word address to EEPROM
  getsSPI( rdptr, length );           // read in multiple bytes
  CS = 1;                             // Deselect Device
  return ( 0 );                       
}

/********************************************************************
*     Function Name:    PutStringSPI                                *
*     Return Value:     void                                        *
*     Parameters:       address of write string storage location    *
*     Description:      This routine writes a string to the SPI bus.*  
********************************************************************/

unsigned char PutStringSPI( unsigned char *wrptr )
{
  unsigned char x;
  for (x = 0; x < PageSize; x++ )     // transmit data until PageSize
  {
     SSPBUF = *wrptr++;               // initiate SPI bus cycle
     while( !SSPSTATbits.BF );        // wait until 'BF' bit is set
  }
  return ( 0 );
}


/********************************************************************
*     Function Name:    SPIWIPPolling                               *
*     Return Value:     void                                        *
*     Parameters:       void                                        *
*     Description:      This routine loops until WIP = 0            *  
********************************************************************/

void SPIWIPPolling(void)
{
  unsigned char Status;
  do
  {
    CS = 0;                           //Select Device
    WriteSPI ( 0x05 );                //Read Status Reg OpCode
    Status = ReadSPI();               //Read Status Reg
    CS = 1;                           //Deselect Device
  } while (Status & 0x01);            //Check for WIP bit Set
}

/********************************************************************
*     Function Name:    WriteEnable                                 *
*     Return Value:     void                                        *
*     Parameters:       void                                        *
*     Description:      This routine sets the Write Enable Latch    *  
********************************************************************/

void WriteEnable(void)
{
    CS = 0;                           //Select Device
    WriteSPI ( 0x06 );                //Write Enable OpCode
    CS = 1;                           //Deselect Device
}
/********************************************************************
*     Function Name:    Read Status                                 *
*     Return Value:     void                                        *
*     Parameters:       void                                        *
*     Description:      This routine sets the Write Enable Latch    *  
********************************************************************/

void ReadStatus(void)
{
    CS = 0;                           //Select Device
    WriteSPI ( 0x05 );                //Read Status Register OpCode
    ReadSPI();                        //Read Status Register
    CS = 1;                           //Deselect Device
}
