/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "typedefs.h"
#include <i2c.h>
#include "usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
//byte old_sw2,old_sw3;
byte counter;
byte trf_state;
//byte temp_mode;
byte moje_data, moje_dataH;

WORD temperature;           // Raw data format
char tempString[10];

unsigned char HighAdd;                    //High Order Address Byte
unsigned char LowAdd;                     //Low Order Address Byte
unsigned char Data;                       //Data Byte
unsigned char Length;                     //Length of bytes to read
unsigned char PageString[64];             // Holds the device page data 
unsigned char counter,counterOff;

DATA_PACKET dataPacket;

//byte pTemp;                     // Pointer to current logging position, will
                                // loop to zero once the max index is reached
//byte valid_temp;                // Keeps count of the valid data points
//word temp_data[30];             // 30 points of data

// Timer0 - 1 second interval setup.
// Fosc/4 = 12MHz
// Use /256 prescalar, this brings counter freq down to 46,875 Hz
// Timer0 should = 65536 - 46875 = 18661 or 0x48E5
/*#define TIMER0L_VAL         0xE5
#define TIMER0H_VAL         0x48*/
#define button_ok PORTCbits.RC0
#define button_pl PORTBbits.RB5
#define button_mi PORTBbits.RB4
/** P R I V A T E  P R O T O T Y P E S ***************************************/

void BlinkUSBStatus(void);
/*BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);
void ResetTempLog(void);
void ReadPOT(void);*/
void ServiceRequests(void);
void Ulcdt (unsigned char data);
void Ulcdc (unsigned char data);
void UinitLCD(void);

// For board testing purpose only
//void PICDEMFSUSBDemoBoardTest(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
//    mInitAllLEDs();
//    mInitAllSwitches();
//    old_sw2 = sw2;
//    old_sw3 = sw3;
    
//    InitTempSensor();
	OpenI2C(MASTER, SLEW_ON);// Initialize I2C module
	SSPADD = 24; //400kHz Baud clock(9) @16MHz
  	DDRCbits.RC7 = 0;
  	DDRCbits.RC6 = 0;
//	DDRCbits.RC1=0;
//	mInitPOT();
    ADCON2bits.ADFM = 1;   // ADC result right justified
	TRISAbits.TRISA1=0;  
//  ResetTempLog();
//    temp_mode = TEMP_REAL_TIME;
	counterOff=0;    
    /* Init Timer0 for data logging interval (every 1 second) */
//    T0CON = 0b10010111;
    //T0CONbits.T08BIT = 0;       // 16-bit mode
    //T0CONbits.T0CS = 0;         // Select Fosc/4
    //T0CONbits.PSA = 0;          // Assign prescalar (default is /256)
    /* Timer0 is already enabled by default */
}//end UserInit


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    BlinkUSBStatus();
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
	if (PORTCbits.RC2==0) LATCbits.LATC1=0;
    ServiceRequests();
}//end ProcessIO

/*void ResetTempLog(void)
{
    pTemp = 0;
    valid_temp = 0;
}//end ResetLog

void ReadPOT(void)
{
    ADCON0bits.GO = 1;              // Start AD conversion
    while(ADCON0bits.NOT_DONE);     // Wait for conversion
    return;
}//end ReadPOT*/

void ServiceRequests(void)
{
    byte index;
    
    if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))
    {
        counter = 0;
        switch(dataPacket.CMD)
        {
            case READ_VERSION:
                //dataPacket._byte[1] is len
                dataPacket._byte[2] = MINOR_VERSION;
                dataPacket._byte[3] = MAJOR_VERSION;
                counter=0x04;
                break;

            case MOJ_REKVEST:
//				HighAdd = 0x00;                         //HighAdd is first byte transmitted for High Density devices
//			  	LowAdd = 0x30;                          //LowAdd is only Address byte for Low Density, second byte to HD
				moje_data=dataPacket._byte[1];
				moje_dataH=dataPacket._byte[2];
				ESeqRead(0xA0, moje_dataH, moje_data, PageString, 32);
//				Delay10KTCYx(250);
//				Delay10KTCYx(250);

                dataPacket._byte[0] = PageString[0];
                dataPacket._byte[1] = PageString[1];
                dataPacket._byte[2] = PageString[2];
                dataPacket._byte[3] = PageString[3];
                dataPacket._byte[4] = PageString[4];
                dataPacket._byte[5] = PageString[5];
                dataPacket._byte[6] = PageString[6];
                dataPacket._byte[7] = PageString[7];
                dataPacket._byte[8] = PageString[8];
                dataPacket._byte[9] = PageString[9];
                dataPacket._byte[10] = PageString[10];
                dataPacket._byte[11] = PageString[11];
                dataPacket._byte[12] = PageString[12];
                dataPacket._byte[13] = PageString[13];
                dataPacket._byte[14] = PageString[14];
                dataPacket._byte[15] = PageString[15];
                dataPacket._byte[16] = PageString[16];
                dataPacket._byte[17] = PageString[17];
                dataPacket._byte[18] = PageString[18];
                dataPacket._byte[19] = PageString[19];
                dataPacket._byte[20] = PageString[20];
                dataPacket._byte[21] = PageString[21];
                dataPacket._byte[22] = PageString[22];
                dataPacket._byte[23] = PageString[23];
                dataPacket._byte[24] = PageString[24];
                dataPacket._byte[25] = PageString[25];
                dataPacket._byte[26] = PageString[26];
                dataPacket._byte[27] = PageString[27];
                dataPacket._byte[28] = PageString[28];
                dataPacket._byte[29] = PageString[29];
                dataPacket._byte[30] = PageString[30];
                dataPacket._byte[31] = PageString[31];



/*                dataPacket._byte[2] = PageString[0];
                dataPacket._byte[3] = PageString[1];
                dataPacket._byte[4] = PageString[2];
                dataPacket._byte[5] = PageString[3];
                dataPacket._byte[6] = PageString[4];
                dataPacket._byte[7] = PageString[5];
                dataPacket._byte[8] = PageString[6];
                dataPacket._byte[9] = PageString[7];
                dataPacket._byte[10] = PageString[8];
                dataPacket._byte[11] = PageString[9];
                dataPacket._byte[12] = PageString[10];
                dataPacket._byte[13] = PageString[11];
                dataPacket._byte[14] = PageString[12];
                dataPacket._byte[15] = PageString[13];
                dataPacket._byte[16] = PageString[14];
                dataPacket._byte[17] = PageString[15];*/
/*              dataPacket._byte[2] = 0x02;
                dataPacket._byte[3] = 0x03;
                dataPacket._byte[4] = 0x06;
                dataPacket._byte[5] = 0x25;*/

	            counter=32;
                break;


                
            default:
                break;
        }//end switch()
        if(counter != 0)
        {
            if(!mUSBGenTxIsBusy())
                USBGenWrite((byte*)&dataPacket,counter);
        }//end if
    }//end if

}//end ServiceRequests

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static long led_count=0;
    
    if(led_count == 0)led_count = 50000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}
	if (led_count==0) 
		{
		if (button_pl==0) counterOff++;
		else counterOff=0;
		}
	if (counterOff>10)
		{
		Ulcdc(0x01);
		Ulcdt(' ');
		Ulcdt(' ');
		Ulcdt('O');
		Ulcdt('F');
		Ulcdt('F');
		Ulcdt(' ');
		LATCbits.LATC1=0;
		}


    if ((UCONbits.SUSPND == 1) & (counter<10))
    {
        if(led_count==0)
        {
   				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('S');
				Ulcdt('P');
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
   				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('D');
				Ulcdt('E');
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
   				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('A');
				Ulcdt('T');
        }
        else if(usb_device_state == POWERED_STATE)
        {
   				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('P');
				Ulcdt('W');
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
   				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('D');
				Ulcdt('F');

        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('A');
				Ulcdt('D');

            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
				Ulcdc(0x01);
				Ulcdt('U');
				Ulcdt('S');
				Ulcdt('B');
				Ulcdt(' ');
				Ulcdt('C');
				Ulcdt('F');
//				mLED_1_Toggle();
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/*BOOL Switch2IsPressed(void)
{
    if(sw2 != old_sw2)
    {
        old_sw2 = sw2;                  // Save new value
        if(sw2 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch2IsPressed

BOOL Switch3IsPressed(void)
{
    if(sw3 != old_sw3)
    {
        old_sw3 = sw3;                  // Save new value
        if(sw3 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch3IsPressed

void TXbyte(byte data)
{
    while(TXSTAbits.TRMT==0);
    TXREG = data;
}//end TXbyte*/

/** EOF user.c ***************************************************************/
